use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::NoTabs 0.15

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'bin/rshasum',
    'lib/App/rshasum.pm',
    't/00-compile.t',
    't/argv.t',
    't/data/1/0.txt',
    't/data/1/2.txt',
    't/data/1/foo/empty',
    't/data/1/zempty',
    't/run.t'
);

notabs_ok($_) foreach @files;
done_testing;
